.\"t
.\" Automatically generated by Pandoc 2.5
.\"
.TH "IBV_WR API" "3" "2018\-11\-27" "libibverbs" "Libibverbs Programmer\[cq]s Manual"
.hy
.SH NAME
.PP
ibv_wr_abort, ibv_wr_complete, ibv_wr_start \- Manage regions allowed to
post work
.PP
ibv_wr_atomic_cmp_swp, ibv_wr_atomic_fetch_add \- Post remote atomic
operation work requests
.PP
ibv_wr_bind_mw, ibv_wr_local_inv \- Post work requests for memory
windows
.PP
ibv_wr_rdma_read, ibv_wr_rdma_write, ibv_wr_rdma_write_imm \- Post RDMA
work requests
.PP
ibv_wr_send, ibv_wr_send_imm, ibv_wr_send_inv \- Post send work requests
.PP
ibv_wr_send_tso \- Post segmentation offload work requests
.PP
ibv_wr_set_inline_data, ibv_wr_set_inline_data_list \- Attach inline
data to the last work request
.PP
ibv_wr_set_sge, ibv_wr_set_sge_list \- Attach data to the last work
request
.PP
ibv_wr_set_ud_addr \- Attach UD addressing info to the last work request
.PP
ibv_wr_set_xrc_srqn \- Attach an XRC SRQN to the last work request
.SH SYNOPSIS
.IP
.nf
\f[C]
#include <infiniband/verbs.h>

void ibv_wr_abort(struct ibv_qp_ex *qp);
int ibv_wr_complete(struct ibv_qp_ex *qp);
void ibv_wr_start(struct ibv_qp_ex *qp);

void ibv_wr_atomic_cmp_swp(struct ibv_qp_ex *qp, uint32_t rkey,
                           uint64_t remote_addr, uint64_t compare,
                           uint64_t swap);
void ibv_wr_atomic_fetch_add(struct ibv_qp_ex *qp, uint32_t rkey,
                             uint64_t remote_addr, uint64_t add);

void ibv_wr_bind_mw(struct ibv_qp_ex *qp, struct ibv_mw *mw, uint32_t rkey,
                    const struct ibv_mw_bind_info *bind_info);
void ibv_wr_local_inv(struct ibv_qp_ex *qp, uint32_t invalidate_rkey);

void ibv_wr_rdma_read(struct ibv_qp_ex *qp, uint32_t rkey,
                      uint64_t remote_addr);
void ibv_wr_rdma_write(struct ibv_qp_ex *qp, uint32_t rkey,
                       uint64_t remote_addr);
void ibv_wr_rdma_write_imm(struct ibv_qp_ex *qp, uint32_t rkey,
                           uint64_t remote_addr, __be32 imm_data);

void ibv_wr_send(struct ibv_qp_ex *qp);
void ibv_wr_send_imm(struct ibv_qp_ex *qp, __be32 imm_data);
void ibv_wr_send_inv(struct ibv_qp_ex *qp, uint32_t invalidate_rkey);
void ibv_wr_send_tso(struct ibv_qp_ex *qp, void *hdr, uint16_t hdr_sz,
                     uint16_t mss);

void ibv_wr_set_inline_data(struct ibv_qp_ex *qp, void *addr, size_t length);
void ibv_wr_set_inline_data_list(struct ibv_qp_ex *qp, size_t num_buf,
                                 const struct ibv_data_buf *buf_list);
void ibv_wr_set_sge(struct ibv_qp_ex *qp, uint32_t lkey, uint64_t addr,
                    uint32_t length);
void ibv_wr_set_sge_list(struct ibv_qp_ex *qp, size_t num_sge,
                         const struct ibv_sge *sg_list);

void ibv_wr_set_ud_addr(struct ibv_qp_ex *qp, struct ibv_ah *ah,
                        uint32_t remote_qpn, uint32_t remote_qkey);
void ibv_wr_set_xrc_srqn(struct ibv_qp_ex *qp, uint32_t remote_srqn);
\f[R]
.fi
.SH DESCRIPTION
.PP
The verbs work request API (ibv_wr_*) allows efficient posting of work
to a send queue using function calls instead of the struct based
\f[I]ibv_post_send()\f[R] scheme.
This approach is designed to minimize CPU branching and locking during
the posting process.
.PP
This API is intended to be used to access additional functionality
beyond what is provided by \f[I]ibv_post_send()\f[R].
.PP
WRs batches of \f[I]ibv_post_send()\f[R] and this API WRs batches can
interleave together just if they are not posted within the critical
region of each other.
(A critical region in this API formed by \f[I]ibv_wr_start()\f[R] and
\f[I]ibv_wr_complete()\f[R]/\f[I]ibv_wr_abort()\f[R])
.SH USAGE
.PP
To use these APIs the QP must be created using ibv_create_qp_ex() which
allows setting the \f[B]IBV_QP_INIT_ATTR_SEND_OPS_FLAGS\f[R] in
\f[I]comp_mask\f[R].
The \f[I]send_ops_flags\f[R] should be set to the OR of the work request
types that will be posted to the QP.
.PP
If the QP does not support all the requested work request types then QP
creation will fail.
.PP
Posting work requests to the QP is done within the critical region
formed by \f[I]ibv_wr_start()\f[R] and
\f[I]ibv_wr_complete()\f[R]/\f[I]ibv_wr_abort()\f[R] (see CONCURRENCY
below).
.PP
Each work request is created by calling a WR builder function (see the
table column WR builder below) to start creating the work request,
followed by allowed/required setter functions described below.
.PP
The WR builder and setter combination can be called multiple times to
efficiently post multiple work requests within a single critical region.
.PP
Each WR builder will use the \f[I]wr_id\f[R] member of \f[I]struct
ibv_qp_ex\f[R] to set the value to be returned in the completion.
Some operations will also use the \f[I]wr_flags\f[R] member to influence
operation (see Flags below).
These values should be set before invoking the WR builder function.
.PP
For example a simple send could be formed as follows:
.IP
.nf
\f[C]
qpx\->wr_id = 1;
ibv_wr_send(qpx);
ibv_wr_set_sge(qpx, lkey, &data, sizeof(data));
\f[R]
.fi
.PP
The section WORK REQUESTS describes the various WR builders and setters
in details.
.PP
Posting work is completed by calling \f[I]ibv_wr_complete()\f[R] or
\f[I]ibv_wr_abort()\f[R].
No work is executed to the queue until \f[I]ibv_wr_complete()\f[R]
returns success.
\f[I]ibv_wr_abort()\f[R] will discard all work prepared since
\f[I]ibv_wr_start()\f[R].
.SH WORK REQUESTS
.PP
Many of the operations match the opcodes available for
\f[I]ibv_post_send()\f[R].
Each operation has a WR builder function, a list of allowed setters, and
a flag bit to request the operation with \f[I]send_ops_flags\f[R] in
\f[I]struct ibv_qp_init_attr_ex\f[R] (see the EXAMPLE below).
.PP
.TS
tab(@);
l l l l.
T{
Operation
T}@T{
WR builder
T}@T{
QP Type Supported
T}@T{
setters
T}
_
T{
ATOMIC_CMP_AND_SWP
T}@T{
ibv_wr_atomic_cmp_swp()
T}@T{
RC, XRC_SEND
T}@T{
DATA, QP
T}
T{
ATOMIC_FETCH_AND_ADD
T}@T{
ibv_wr_atomic_fetch_add()
T}@T{
RC, XRC_SEND
T}@T{
DATA, QP
T}
T{
BIND_MW
T}@T{
ibv_wr_bind_mw()
T}@T{
UC, RC, XRC_SEND
T}@T{
NONE
T}
T{
LOCAL_INV
T}@T{
ibv_wr_local_inv()
T}@T{
UC, RC, XRC_SEND
T}@T{
NONE
T}
T{
RDMA_READ
T}@T{
ibv_wr_rdma_read()
T}@T{
RC, XRC_SEND
T}@T{
DATA, QP
T}
T{
RDMA_WRITE
T}@T{
ibv_wr_rdma_write()
T}@T{
UC, RC, XRC_SEND
T}@T{
DATA, QP
T}
T{
RDMA_WRITE_WITH_IMM
T}@T{
ibv_wr_rdma_write_imm()
T}@T{
UC, RC, XRC_SEND
T}@T{
DATA, QP
T}
T{
SEND
T}@T{
ibv_wr_send()
T}@T{
UD, UC, RC, XRC_SEND, RAW_PACKET
T}@T{
DATA, QP
T}
T{
SEND_WITH_IMM
T}@T{
ibv_wr_send_imm()
T}@T{
UD, UC, RC, SRC SEND
T}@T{
DATA, QP
T}
T{
SEND_WITH_INV
T}@T{
ibv_wr_send_inv()
T}@T{
UC, RC, XRC_SEND
T}@T{
DATA, QP
T}
T{
TSO
T}@T{
ibv_wr_send_tso()
T}@T{
UD, RAW_PACKET
T}@T{
DATA, QP
T}
.TE
.SS Atomic operations
.PP
Atomic operations are only atomic so long as all writes to memory go
only through the same RDMA hardware.
It is not atomic with writes performed by the CPU, or by other RDMA
hardware in the system.
.TP
.B \f[I]ibv_wr_atomic_cmp_swp()\f[R]
If the remote 64 bit memory location specified by \f[I]rkey\f[R] and
\f[I]remote_addr\f[R] equals \f[I]compare\f[R] then set it to
\f[I]swap\f[R].
.TP
.B \f[I]ibv_wr_atomic_fetch_add()\f[R]
Add \f[I]add\f[R] to the 64 bit memory location specified \f[I]rkey\f[R]
and \f[I]remote_addr\f[R].
.SS Memory Windows
.PP
Memory window type 2 operations (See man page for ibv_alloc_mw).
.TP
.B \f[I]ibv_wr_bind_mw()\f[R]
Bind a MW type 2 specified by \f[B]mw\f[R], set a new \f[B]rkey\f[R] and
set its properties by \f[B]bind_info\f[R].
.TP
.B \f[I]ibv_wr_local_inv()\f[R]
Invalidate a MW type 2 which is associated with \f[B]rkey\f[R].
.SS RDMA
.TP
.B \f[I]ibv_wr_rdma_read()\f[R]
Read from the remote memory location specified \f[I]rkey\f[R] and
\f[I]remote_addr\f[R].
The number of bytes to read, and the local location to store the data,
is determined by the DATA buffers set after this call.
.TP
.B \f[I]ibv_wr_rdma_write()\f[R], \f[I]ibv_wr_rdma_write_imm()\f[R]
Write to the remote memory location specified \f[I]rkey\f[R] and
\f[I]remote_addr\f[R].
The number of bytes to read, and the local location to get the data, is
determined by the DATA buffers set after this call.
.RS
.PP
The _imm version causes the remote side to get a
IBV_WC_RECV_RDMA_WITH_IMM containing the 32 bits of immediate data.
.RE
.SS Message Send
.TP
.B \f[I]ibv_wr_send()\f[R], \f[I]ibv_wr_send_imm()\f[R]
Send a message.
The number of bytes to send, and the local location to get the data, is
determined by the DATA buffers set after this call.
.RS
.PP
The _imm version causes the remote side to get a
IBV_WC_RECV_RDMA_WITH_IMM containing the 32 bits of immediate data.
.RE
.TP
.B \f[I]ibv_wr_send_inv()\f[R]
The data transfer is the same as for \f[I]ibv_wr_send()\f[R], however
the remote side will invalidate the MR specified by
\f[I]invalidate_rkey\f[R] before delivering a completion.
.TP
.B \f[I]ibv_wr_send_tso()\f[R]
Produce multiple SEND messages using TCP Segmentation Offload.
The SGE points to a TCP Stream buffer which will be segmented into MSS
size SENDs.
The hdr includes the entire network headers up to and including the TCP
header and is prefixed before each segment.
.SS QP Specific setters
.PP
Certain QP types require each post to be accompanied by additional
setters, these setters are mandatory for any operation listing a QP
setter in the above table.
.TP
.B \f[I]UD\f[R] QPs
\f[I]ibv_wr_set_ud_addr()\f[R] must be called to set the destination
address of the work.
.TP
.B \f[I]XRC_SEND\f[R] QPs
\f[I]ibv_wr_set_xrc_srqn()\f[R] must be called to set the destination
SRQN field.
.SS DATA transfer setters
.PP
For work that requires to transfer data one of the following setters
should be called once after the WR builder:
.TP
.B \f[I]ibv_wr_set_sge()\f[R]
Transfer data to/from a single buffer given by the lkey, addr and
length.
This is equivalent to \f[I]ibv_wr_set_sge_list()\f[R] with a single
element.
.TP
.B \f[I]ibv_wr_set_sge_list()\f[R]
Transfer data to/from a list of buffers, logically concatenated
together.
Each buffer is specified by an element in an array of \f[I]struct
ibv_sge\f[R].
.PP
Inline setters will copy the send data during the setter and allows the
caller to immediately re\-use the buffer.
This behavior is identical to the IBV_SEND_INLINE flag.
Generally this copy is done in a way that optimizes SEND latency and is
suitable for small messages.
The provider will limit the amount of data it can support in a single
operation.
This limit is requested in the \f[I]max_inline_data\f[R] member of
\f[I]struct ibv_qp_init_attr\f[R].
Valid only for SEND and RDMA_WRITE.
.TP
.B \f[I]ibv_wr_set_inline_data()\f[R]
Copy send data from a single buffer given by the addr and length.
This is equivalent to \f[I]ibv_wr_set_inline_data_list()\f[R] with a
single element.
.TP
.B \f[I]ibv_wr_set_inline_data_list()\f[R]
Copy send data from a list of buffers, logically concatenated together.
Each buffer is specified by an element in an array of \f[I]struct
ibv_inl_data\f[R].
.SS Flags
.PP
A bit mask of flags may be specified in \f[I]wr_flags\f[R] to control
the behavior of the work request.
.TP
.B \f[B]IBV_SEND_FENCE\f[R]
Do not start this work request until prior work has completed.
.TP
.B \f[B]IBV_SEND_IP_CSUM\f[R]
Offload the IPv4 and TCP/UDP checksum calculation
.TP
.B \f[B]IBV_SEND_SIGNALED\f[R]
A completion will be generated in the completion queue for the
operation.
.TP
.B \f[B]IBV_SEND_SOLICTED\f[R]
Set the solicted bit in the RDMA packet.
This informs the other side to generate a completion event upon
receiving the RDMA operation.
.SH CONCURRENCY
.PP
The provider will provide locking to ensure that
\f[I]ibv_wr_start()\f[R] and \f[I]ibv_wr_complete()/abort()\f[R] form a
per\-QP critical section where no other threads can enter.
.PP
If an \f[I]ibv_td\f[R] is provided during QP creation then no locking
will be performed and it is up to the caller to ensure that only one
thread can be within the critical region at a time.
.SH RETURN VALUE
.PP
Applications should use this API in a way that does not create failures.
The individual APIs do not return a failure indication to avoid
branching.
.PP
If a failure is detected during operation, for instance due to an
invalid argument, then \f[I]ibv_wr_complete()\f[R] will return failure
and the entire posting will be aborted.
.SH EXAMPLE
.IP
.nf
\f[C]
/* create RC QP type and specify the required send opcodes */
qp_init_attr_ex.qp_type = IBV_QPT_RC;
qp_init_attr_ex.comp_mask |= IBV_QP_INIT_ATTR_SEND_OPS_FLAGS;
qp_init_attr_ex.send_ops_flags |= IBV_QP_EX_WITH_RDMA_WRITE;
qp_init_attr_ex.send_ops_flags |= IBV_QP_EX_WITH_RDMA_WRITE_WITH_IMM;

ibv_qp *qp = ibv_create_qp_ex(ctx, qp_init_attr_ex);
ibv_qp_ex *qpx = ibv_qp_to_qp_ex(qp);

ibv_wr_start(qpx);

/* create 1st WRITE WR entry */
qpx\->wr_id = my_wr_id_1;
ibv_wr_rdma_write(qpx, rkey, remote_addr_1);
ibv_wr_set_sge(qpx, lkey, local_addr_1, length_1);

/* create 2nd WRITE_WITH_IMM WR entry */
qpx\->wr_id = my_wr_id_2;
qpx\->send_flags = IBV_SEND_SIGNALED;
ibv_wr_rdma_write_imm(qpx, rkey, remote_addr_2, htonl(0x1234));
ibv_set_wr_sge(qpx, lkey, local_addr_2, length_2);

/* Begin processing WRs */
ret = ibv_wr_complete(qpx);
\f[R]
.fi
.SH SEE ALSO
.PP
\f[B]ibv_post_send\f[R](3), \f[B]ibv_create_qp_ex(3)\f[R].
.SH AUTHOR
.PP
Jason Gunthorpe <jgg@mellanox.com> Guy Levi <guyle@mellanox.com>
